import argparse
# import os
from glpic import Glpi
from fastmcp import FastMCP, Context
from fastmcp.server.dependencies import get_http_headers as h

mcp = FastMCP("glpimcp")


@mcp.tool()
def create_reservation(context: Context,
                       user: str, computer: str, overrides: dict = {}) -> dict:
    """Create glpi reservation"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.create_reservation(user, computer, overrides)


@mcp.tool()
def delete_reservation(context: Context,
                       reservation: str) -> dict:
    """Delete glpi reservation"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.delete_reservation(reservation)


@mcp.tool()
def info_computer(context: Context,
                  computer: str, overrides: dict = {}) -> dict:
    """Get info on glpi computer"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.info_computer(computer)


@mcp.tool()
def info_reservation(context: Context,
                     reservation: str) -> dict:
    """Get info on glpi reservation"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.info_reservation(reservation)


@mcp.tool()
def get_user(context: Context,
             user: str) -> dict:
    """Get info on glpi user"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.get_user(user)


@mcp.tool()
def list_computers(context: Context,
                   overrides: dict) -> list:
    """List glpi computers"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.list_computers(overrides)


@mcp.tool()
def list_reservations(context: Context,
                      user: str) -> list:
    """List glpi reservations"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.list_reservations(user)


@mcp.tool()
def list_users(context: Context,
               overrides: dict = {}) -> list:
    """List glpi users"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.list_users(overrides)


@mcp.tool()
def update_computer(context: Context,
                    computer: str, overrides: dict) -> dict:
    """Update glpi computer"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.update_computer(computer, overrides)


@mcp.tool()
def update_reservation(context: Context,
                       user: str, reservation: str, overrides: dict = {}) -> dict:
    """Create glpi reservation"""
    glpic = Glpi(h.get('glpi_url'), h.get('glpi_user'), h.get('glpi_token'))
    return glpic.update_reservation(user, reservation, overrides)


def main():
    parser = argparse.ArgumentParser(description="glpimcp")
    parser.add_argument("--port", type=int, default=8000, help="Localhost port to listen on")
    parser.add_argument("-s", "--stdio", action='store_true')
    args = parser.parse_args()
    parameters = {'transport': 'stdio'} if args.stdio else {'transport': 'http', 'host': '0.0.0.0', 'port': args.port}
    mcp.run(**parameters)


if __name__ == "__main__":
    main()



import argparse
from argparse import RawDescriptionHelpFormatter as rawhelp
from glpic import Glpi
from glpic import error, handle_parameters, info
import os
from prettytable import PrettyTable
import sys
from textwrap import fill

PARAMHELP = "specify parameter or keyword for rendering (multiple can be specified)"


def confirm(message):
    message = f"{message} [y/N]: "
    try:
        _input = input(message)
        if _input.lower() not in ['y', 'yes']:
            error("Leaving...")
            sys.exit(1)
    except:
        sys.exit(1)
    return


def container_path(path):
    if os.path.exists('/i_am_a_container'):
        if path == '.':
            return '/workdir'
        elif not os.path.isabs(path):
            return f'/workdir/{path}'
    return path


def get_subparser_print_help(parser, subcommand):
    subparsers_actions = [
        action for action in parser._actions
        if isinstance(action, argparse._SubParsersAction)]
    for subparsers_action in subparsers_actions:
        for choice, subparser in subparsers_action.choices.items():
            if choice == subcommand:
                subparser.print_help()
                return


def get_subparser(parser, subcommand):
    subparsers_actions = [
        action for action in parser._actions
        if isinstance(action, argparse._SubParsersAction)]
    for subparsers_action in subparsers_actions:
        for choice, subparser in subparsers_action.choices.items():
            if choice == subcommand:
                return subparser


def create_reservation(args):
    glpic = Glpi(args.url, args.user, args.token)
    overrides = handle_parameters(args.param)
    user = overrides.get('user') or glpic.user
    computer = args.computer or overrides.get('computer')
    if computer is None:
        error("Missing computer")
        sys.exit(1)
    info(f"Creating reservation for computer {computer} and for user {user}")
    glpic.create_reservation(user, computer, overrides)


def delete_reservation(args):
    yes = args.yes
    yes_top = args.yes_top
    if not yes and not yes_top:
        confirm("Are you sure?")
    glpic = Glpi(args.url, args.user, args.token)
    for reservation in args.reservations:
        info(f"Deleting reservation {reservation}")
        glpic.delete_reservation(reservation)


def update_reservation(args):
    glpic = Glpi(args.url, args.user, args.token)
    overrides = handle_parameters(args.param)
    user = overrides.get('user') or glpic.user
    reservations = args.reservations
    if not reservations:
        reservations = [r['id'] for r in glpic.list_reservations(user)]
    for reservation in reservations:
        info(f"Updating reservation {reservation}")
        glpic.update_reservation(user, reservation, overrides=overrides)


def info_computer(args):
    glpic = Glpi(args.url, args.user, args.token)
    overrides = {'computer': args.computer} if args.computer is not None else {}
    overrides.update(handle_parameters(args.param))
    data = glpic.info_computer(overrides)
    for computer in data:
        for key in computer:
            print(f"{key}: {computer[key]}")
        print('-----------------')


def info_reservation(args):
    glpic = Glpi(args.url, args.user, args.token)
    data = glpic.info_reservation(args.reservation)
    for key in data:
        print(f"{key}: {data[key]}")


def list_computers(args):
    glpic = Glpi(args.url, args.user, args.token)
    computerstable = PrettyTable(["Name", "Group", "Serial", "Model", "Memory", "Bmc"])
    for computer in glpic.list_computers(overrides=handle_parameters(args.param)):
        name, serial = computer['Computer.name'], computer['Computer.serial']
        group, memory = computer['Computer.Group.completename'], computer.get('Computer.Item_DeviceMemory.size')
        bmc = computer['Computer.PluginFieldsComputerbmcaddre.bmcaddressfield']
        model = computer['Computer.ComputerModel.name']
        entry = [name, group, serial, model, memory, bmc]
        computerstable.add_row(entry)
    print(computerstable)


def update_computer(args):
    glpic = Glpi(args.url, args.user, args.token)
    for computer in args.computers:
        info(f"Updating computer {computer}")
        glpic.update_computer(computer, overrides=handle_parameters(args.param))


def list_reservations(args):
    glpic = Glpi(args.url, args.user, args.token)
    overrides = handle_parameters(args.param)
    user = overrides.get('user') or glpic.user
    reservationstable = PrettyTable(["Id", "Item", "Begin", "End", "Comment"])
    for reservation in glpic.list_reservations(user):
        _id, begin, end, comment = reservation['id'], reservation['begin'], reservation['end'], reservation['comment']
        comment = fill(comment, width=100)
        reservation_id = reservation['reservationitems_id']
        computer_id = glpic.info_reservation(reservation_id)['items_id']
        reservation_name = glpic.info_computer({'computer': computer_id})[0]['Computer.name']
        entry = [_id, reservation_name, begin, end, comment]
        reservationstable.add_row(entry)
    print(reservationstable)


def list_users(args):
    glpic = Glpi(args.url, args.user, args.token)
    userstable = PrettyTable(["Id", "Name", "Last Login"])
    for user in glpic.list_users(overrides=handle_parameters(args.param)):
        _id, name, last_login = user['id'], user['name'], user['last_login']
        entry = [_id, name, last_login]
        userstable.add_row(entry)
    print(userstable)


def cli():
    """

    """
    # PARAMETERS_HELP = 'specify parameter or keyword for rendering (multiple can be specified)'
    parser = argparse.ArgumentParser(description='Glpi client')
    parser.add_argument('-t', '--token')
    parser.add_argument('-u', '-U', '--url')
    parser.add_argument('-user')
    subparsers = parser.add_subparsers(metavar='', title='Available Commands')

    create_desc = 'Create Object'
    create_parser = subparsers.add_parser('create', description=create_desc, help=create_desc, aliases=['add'])
    create_subparsers = create_parser.add_subparsers(metavar='', dest='subcommand_create')

    reservationcreate_desc = 'Create Reservation'
    reservationcreate_epilog = None
    reservationcreate_parser = create_subparsers.add_parser('reservation', description=reservationcreate_desc,
                                                            help=reservationcreate_desc,
                                                            epilog=reservationcreate_epilog, formatter_class=rawhelp)
    reservationcreate_parser.add_argument('-f', '--force', action='store_true',
                                          help='Delete existing reservation if needed')
    reservationcreate_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    reservationcreate_parser.add_argument('computer', metavar='COMPUTER', nargs='?')
    reservationcreate_parser.set_defaults(func=create_reservation)

    delete_desc = 'Delete Object'
    delete_parser = subparsers.add_parser('delete', description=delete_desc, help=delete_desc, aliases=['remove'])
    delete_parser.add_argument('-y', '--yes', action='store_true', help='Dont ask for confirmation', dest="yes_top")
    delete_subparsers = delete_parser.add_subparsers(metavar='', dest='subcommand_delete')

    reservationdelete_desc = 'Delete Reservation'
    reservationdelete_epilog = None
    reservationdelete_parser = delete_subparsers.add_parser('reservation', description=reservationdelete_desc,
                                                            help=reservationdelete_desc,
                                                            epilog=reservationdelete_epilog, formatter_class=rawhelp,
                                                            aliases=['reservations'])
    reservationdelete_parser.add_argument('-a', '--all', action='store_true', help='Delete all reservations')
    reservationdelete_parser.add_argument('-y', '--yes', action='store_true', help='Dont ask for confirmation')
    reservationdelete_parser.add_argument('reservations', metavar='CLUSTERS', nargs='*')
    reservationdelete_parser.set_defaults(func=delete_reservation)

    info_desc = 'Info Object'
    info_parser = subparsers.add_parser('info', description=info_desc, help=info_desc)
    info_subparsers = info_parser.add_subparsers(metavar='', dest='subcommand_info')

    computerinfo_desc = 'Info Computer'
    computerinfo_epilog = None
    computerinfo_parser = info_subparsers.add_parser('computer', description=computerinfo_desc, help=computerinfo_desc,
                                                     epilog=computerinfo_epilog, formatter_class=rawhelp)
    computerinfo_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    computerinfo_parser.add_argument('computer', metavar='COMPUTER', nargs='?')
    computerinfo_parser.set_defaults(func=info_computer)

    reservationinfo_desc = 'Info Reservation'
    reservationinfo_epilog = None
    reservationinfo_parser = info_subparsers.add_parser('reservation', description=reservationinfo_desc,
                                                        help=reservationinfo_desc,
                                                        epilog=reservationinfo_epilog, formatter_class=rawhelp)
    reservationinfo_parser.add_argument('reservation', metavar='RESERVATION')
    reservationinfo_parser.set_defaults(func=info_reservation)

    list_desc = 'List Object'
    list_parser = subparsers.add_parser('list', description=list_desc, help=list_desc, aliases=['get'])
    list_subparsers = list_parser.add_subparsers(metavar='', dest='subcommand_list')

    computerlist_desc = 'List Computers'
    computerlist_parser = argparse.ArgumentParser(add_help=False)
    computerlist_parser.set_defaults(func=list_computers)
    computerlist_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    list_subparsers.add_parser('computer', parents=[computerlist_parser], description=computerlist_desc,
                               help=computerlist_desc, aliases=['computers'])

    reservationlist_desc = 'List Reservations'
    reservationlist_parser = argparse.ArgumentParser(add_help=False)
    reservationlist_parser.set_defaults(func=list_reservations)
    reservationlist_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    list_subparsers.add_parser('reservation', parents=[reservationlist_parser], description=reservationlist_desc,
                               help=reservationlist_desc, aliases=['reservations'])

    userlist_desc = 'List Users'
    userlist_parser = argparse.ArgumentParser(add_help=False)
    userlist_parser.set_defaults(func=list_users)
    userlist_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    list_subparsers.add_parser('user', parents=[userlist_parser], description=userlist_desc,
                               help=userlist_desc, aliases=['users'])

    update_desc = 'Update Object'
    update_parser = subparsers.add_parser('update', description=update_desc, help=update_desc)
    update_subparsers = update_parser.add_subparsers(metavar='', dest='subcommand_update')

    computerupdate_desc = 'Update Computer'
    computerupdate_epilog = None
    computerupdate_parser = update_subparsers.add_parser('computer', description=computerupdate_desc,
                                                         help=computerupdate_desc,
                                                         epilog=computerupdate_epilog, formatter_class=rawhelp,
                                                         aliases=['computers'])
    computerupdate_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    computerupdate_parser.add_argument('computers', metavar='RESERVATIONS', nargs='*')
    computerupdate_parser.set_defaults(func=update_computer)

    reservationupdate_desc = 'Update Reservation'
    reservationupdate_epilog = None
    reservationupdate_parser = update_subparsers.add_parser('reservation', description=reservationupdate_desc,
                                                            help=reservationupdate_desc,
                                                            epilog=reservationupdate_epilog, formatter_class=rawhelp,
                                                            aliases=['reservations'])
    reservationupdate_parser.add_argument('-P', '--param', action='append', help=PARAMHELP, metavar='PARAM')
    reservationupdate_parser.add_argument('reservations', metavar='RESERVATIONS', nargs='*')
    reservationupdate_parser.set_defaults(func=update_reservation)

    if len(sys.argv) == 1:
        parser.print_help()
        os._exit(0)
    args = parser.parse_args()
    if not hasattr(args, 'func'):
        for attr in dir(args):
            if attr.startswith('subcommand_') and getattr(args, attr) is None:
                split = attr.split('_')
                if len(split) == 2:
                    subcommand = split[1]
                    get_subparser_print_help(parser, subcommand)
                elif len(split) == 3:
                    subcommand = split[1]
                    subsubcommand = split[2]
                    subparser = get_subparser(parser, subcommand)
                    get_subparser_print_help(subparser, subsubcommand)
                os._exit(0)
        os._exit(0)
    args.func(args)


if __name__ == '__main__':
    cli()



from ast import literal_eval
from datetime import date, datetime, timedelta
import json
import ssl
import os
import sys
from urllib.request import urlopen, Request
from urllib.parse import urlencode


def parse_date(date):
    date = str(date)
    formats = ["%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H:%M", "%Y-%m-%d"]
    for fmt in formats:
        try:
            correct_date = datetime.strptime(date, fmt)
            return correct_date.strftime(fmt)
        except ValueError:
            pass
    raise ValueError(f"Unsupported date format: {date}")


def handle_parameters(parameters):
    overrides = {}
    processed_keys = []
    if parameters is None:
        return {}
    for parameter in parameters:
        if len(parameter.split('=')) < 2:
            error(f"Wrong parameter {parameter}. Should be key=value")
            sys.exit(1)
        else:
            if len(parameter.split('=')) == 2:
                key, value = parameter.split('=')
            else:
                split = parameter.split('=')
                key = split[0]
                value = parameter.replace(f"{key}=", '')
            if key in processed_keys:
                error(f"Repeated parameter {key}")
                sys.exit(1)
            else:
                processed_keys.append(key)
            if value.isdigit():
                value = int(value)
            elif value.lower() == 'true':
                value = True
            elif value.lower() == 'false':
                value = False
            elif value == 'None':
                value = None
            elif value == '[]':
                value = []
            elif value.startswith('[') and value.endswith(']'):
                if '{' in value:
                    value = literal_eval(value)
                else:
                    value = value[1:-1].split(',')
                    for index, v in enumerate(value):
                        v = v.strip()
                        value[index] = v
            overrides[key] = value
    return overrides


def _delete(url, headers):
    request = Request(url, headers=headers, method='DELETE')
    try:
        return urlopen(request)
    except Exception as e:
        error(e.read().decode())


def _get(url, headers):
    if not os.path.basename(url).split('?')[0].isnumeric():
        encoded_params = urlencode({"range": '0-99999'})
        delimiter = '&' if '?' in url else '?'
        url += f'{delimiter}{encoded_params}'
    try:
        return json.loads(urlopen(Request(url, headers=headers)).read())
    except Exception as e:
        error(e.read().decode())


def _patch(url, headers, data):
    data = json.dumps(data).encode('utf-8')
    try:
        return urlopen(Request(url, data=data, headers=headers, method='PATCH'))
    except Exception as e:
        error(e.read().decode())


def _post(url, headers, data):
    data = json.dumps(data).encode('utf-8')
    try:
        return urlopen(Request(url, data=data, headers=headers, method='POST'))
    except Exception as e:
        error(e.read().decode())


def _put(url, headers, data):
    data = json.dumps(data).encode('utf-8')
    try:
        return urlopen(Request(url, data=data, headers=headers, method='PUT'))
    except Exception as e:
        error(e.read().decode())


def error(text):
    color = "31"
    print(f'\033[0;{color}m{text}\033[0;0m')


def warning(text):
    color = "33"
    print(f'\033[0;{color}m{text}\033[0;0m')


def info(text):
    color = "36"
    print(f'\033[0;{color}m{text}\033[0;0m')


class Glpi(object):
    def __init__(self, url, user, token):
        self.url = url or os.environ.get('GLPI_URL')
        if self.url is None:
            error('GLPI_URL is not set')
            sys.exit(1)
        user = user or os.environ.get('GLPI_USER')
        if user is None:
            error('GLPI_USER is not set')
            sys.exit(1)
        self.user = user.split('@')[0]
        token = token or os.environ.get('GLPI_TOKEN')
        if token is None:
            error('GLPI_TOKEN is not set')
            sys.exit(1)
        ssl._create_default_https_context = ssl._create_unverified_context
        headers = {'Content-Type': 'application/json', 'Authorization': f"user_token {token}"}
        response = _get(f'{self.url}/initSession?get_full_session=true', headers)
        self.headers = {'Content-Type': 'application/json', "Session-Token": response['session_token']}

    def get_user(self, user=None):
        user = user or self.user
        users = _get(f'{self.url}/User', headers=self.headers)
        for u in users:
            if user in u['name']:
                return u

    def list_users(self, overrides={}):
        users = []
        for u in _get(f'{self.url}/User', headers=self.headers):
            if overrides:
                for key in overrides:
                    if u[key] != overrides[key]:
                        continue
            users.append(u)
        return users

    def get_options(self, item_type):
        search_options = {}
        all_options = _get(f'{self.url}/listSearchOptions/{item_type}', headers=self.headers)
        for key in all_options:
            if key.isnumeric():
                search_options[all_options[key]['uid']] = key
        return search_options

    def info_computer(self, overrides={}):
        computer = overrides.get('computer')
        if computer is not None:
            field = 2 if isinstance(computer, int) or computer.isnumeric() else 1
            search_data = "criteria[0][link]=AND&criteria[0][itemtype]=Computer"
            search_data = f"&criteria[0][field]={field}&criteria[0][searchtype]=contains"
            search_data += f"&criteria[0][value]={computer}"
        else:
            search_options = self.get_options('Computer')
            search_data = ''
            for index, key in enumerate(overrides):
                value = overrides[key]
                if not key.startswith('Computer'):
                    key = f'Computer.{key}'
                if key not in search_options:
                    warning("Invalid key {key}")
                    continue
                key_id = search_options[key]
                search_data += "criteria[{index}][link]=AND&criteria[{index}][itemtype]=Computer"
                search_data = f"&criteria[{index}][field]={key_id}&criteria[{index}][searchtype]=contains"
                search_data += f"&criteria[{index}][value]={value}"
        url = f'{self.url}/search/Computer?{search_data}&uid_cols'
        if overrides.get('uid', False):
            url += '&forcedisplay[0]=2'
        computers = _get(url, headers=self.headers)
        return computers['data'] if 'data' in computers else []

    def info_reservation(self, reservation):
        return _get(f'{self.url}/ReservationItem/{reservation}', headers=self.headers)

    def list_reservations(self, user):
        now = datetime.now()
        response = _get(f'{self.url}/Reservation', headers=self.headers)
        user_id = self.get_user(user)['id']
        l = [r for r in response if r['users_id'] == user_id and parse_date(r['end']) > now]
        return l

    def list_computers(self, user=None, overrides={}):
        computers = _get(f'{self.url}/search/Computer?uid_cols', headers=self.headers)['data']
        if not overrides:
            return computers
        results = []
        memory = overrides.get('memory')
        cpu_model = overrides.get('cpumodel')
        number = overrides.get('number')
        for computer in computers:
            current_cpu_model = computer['Computer.Item_DeviceProcessor.DeviceProcessor.designation'] or 'XXX'
            if isinstance(current_cpu_model, list):
                current_cpu_model = current_cpu_model[0]
            if cpu_model is not None and cpu_model.lower() not in current_cpu_model.lower():
                continue
            current_memory = computer['Computer.Item_DeviceMemory.size'] or '0'
            if memory is not None and int(float(current_memory)) < int(memory):
                continue
            results.append(computer)
            if number is not None and len(results) >= int(number):
                break
        return results

    def update_computer(self, computer, overrides):
        computer_info = self.info_computer({'computer': computer, 'uid': True})
        if not computer_info:
            error(f"Computer {computer} not found")
            return
        computer_id = computer_info[0]['Computer.id']
        valid_keys = list(_get(f'{self.url}/Computer/', self.headers)[0].keys())
        wrong_keys = [key for key in overrides if key not in valid_keys]
        if wrong_keys:
            error(f"Ignoring keys {','.join(wrong_keys)}")
            for key in wrong_keys:
                del overrides[key]
        if not overrides:
            info("Nothing to update")
        data = {'input': overrides}
        return _put(f'{self.url}/Computer/{computer_id}', self.headers, data)

    def create_reservation(self, user, computer, overrides):
        overrides['begin'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        if 'end' not in overrides:
            overrides['end'] = date.today() + timedelta(days=30)
        if 'users_id' not in overrides:
            user_id = self.get_user(user)['id']
            overrides['users_id'] = user_id
        overrides['end'] = parse_date(str(overrides['end']))
        if 'comment' not in overrides:
            overrides['comment'] = f'reservation for {user}'
        computer_id = self.info_computer({'computer': computer, 'uid': True})[0]['Computer.id']
        reservationitem_id = self.get_reservation_item_id(computer_id)
        overrides['reservationitems_id'] = reservationitem_id
        valid_keys = list(_get(f'{self.url}/Reservation/', self.headers)[0].keys())
        wrong_keys = [key for key in overrides if key not in valid_keys]
        if wrong_keys:
            error(f"Ignoring keys {','.join(wrong_keys)}")
            for key in wrong_keys:
                del overrides[key]
        if not overrides:
            info("Nothing to create")
            return
        data = {'input': overrides}
        return _post(f'{self.url}/Reservation', self.headers, data)

    def delete_reservation(self, reservation):
        return _delete(f'{self.url}/Reservation/{reservation}', headers=self.headers)

    def update_reservation(self, user, reservation, overrides={}):
        valid_keys = list(_get(f'{self.url}/Reservation/', self.headers)[0].keys()) + ['user']
        wrong_keys = [key for key in overrides if key not in valid_keys]
        if wrong_keys:
            error(f"Ignoring keys {','.join(wrong_keys)}")
            for key in wrong_keys:
                del overrides[key]
        if not overrides:
            date_after_month = datetime.today() + timedelta(days=30)
            new_date = date_after_month.strftime('%Y-%m-%d 00:00:00')
            warning(f"Setting end date to {new_date}")
            overrides['end'] = new_date
        if 'end' in overrides:
            overrides['end'] = parse_date(str(overrides['end']))
        if str(user).isnumeric():
            overrides['users_id'] = self.get_user(user)['id']
        if 'user' in overrides:
            del overrides['user']
        data = {'input': overrides}
        return _put(f'{self.url}/Reservation/{reservation}', self.headers, data)

    def get_reservation_item_id(self, computer_id):
        url = f'{self.url}/ReservationItem?uid_cols'
        reservation_items = _get(url, headers=self.headers)
        for entry in reservation_items:
            if entry['itemtype'] == 'Computer' and entry['items_id'] == computer_id:
                return entry['id']
